#pragma once
#include <atlcomcli.h>
#include <list>
// Import the type library.
#import "C:\Program Files\SIDRA SOLUTIONS\SIDRA INTERSECTION 8.0\SIDRASolutions.SI.API.tlb" raw_interfaces_only

using namespace SIDRASolutions_SI_API;

namespace CppSample
{
	const long MAXOrientation = 8; // Leg or Movement orientation is from 0 to 7; 0 = South, 1 = South East, 2 = East ...
	const long MAXMcClass = 12; // Movement Class is from 1 to 12; 1 = Light Vehicles, 2 = Heavy Vehicles, 3 = Buses ...
	const long MAXPedCode = 4; // Pedestrian Movement Code is from 0 to 3, it is for saving Leg Ped Mov data in an array; 0 = Full Crossing, 1 = Slip Lane, 2 = Staged1, 3 = Stage2. The Diagnoal Ped Mov is not in the array because one Site only has one Diagonal Ped Mov.
	const long MAXPedType = 4; // Pedestrian Movement Type is from 1 to 4; 1 = Full Crossing, 2 = Slip Lane, 3 = Staged, 4 = Diagonal.
	const long MAXPedStage = 2; // Pedestrian Movement Stage is from 1 to 2;

	class CppSIAPI
	{
	public:
		CppSIAPI();
		~CppSIAPI();
		CComPtr<ISIAPI> get_SIAPI();

		// Find the included Movement Classes in this Site
		int ReadIncludedMCs(CComPtr<ISIAPISite> site, bool includedMCs[MAXMcClass]);

		// Find the existing Veh ODs in this Site
		int ReadExistingVehODs(CComPtr<ISIAPISite> site, bool existingVehODs[MAXOrientation][MAXOrientation]);

		// Find the existing Peds in this Site
		int ReadExistingPeds(CComPtr<ISIAPISite> site, bool *existingPedDiagonal, bool existingPeds[MAXOrientation][MAXPedCode]);

		// Convert Ped Code (in array dimension) to Ped Type and Ped Stage
		int ConvertPedCodeToTypeStage(long pedCode, long *pedType, long *pedStage);

		// Convert Ped Type and Ped Stage to Ped Code (in array dimension) 
		int ConvertPedTypeStageToCode(long pedType, long pedStage, long *pedCode);

		// Read Lane Disciplines on an Approach Lane
		int ReadLaneApprLaneDisciplines(CComPtr<ISIAPILaneApproach> laneAppr, long origin,
			bool existingVehODs[MAXOrientation][MAXOrientation], bool includedMCs[MAXMcClass],
			bool laneApprMovMCExists[MAXOrientation][MAXMcClass]);

		// Read Opposing Veh OD Movement data for all Veh OD Movements in this Site
		int ReadOpposingVehData(CComPtr<ISIAPISite> site,
			bool existingVehODs[MAXOrientation][MAXOrientation],
			bool opposingVehData[MAXOrientation][MAXOrientation][MAXOrientation][MAXOrientation]);

		// Read Opposing Ped Movement data for all Veh OD Movements in this Site
		int ReadOpposingPedData(CComPtr<ISIAPISite> site,
			bool existingVehODs[MAXOrientation][MAXOrientation], bool existingPeds[MAXOrientation][MAXPedCode],
			bool opposingPedData[MAXOrientation][MAXOrientation][MAXOrientation][MAXPedCode]);

		// Set the default Lane Disciplines on all Approach Lanes of a Site
		int SetDefaultLaneDisciplines(CComPtr<ISIAPISite> site);

		int SetLegDefaultLaneDisciplines(CComPtr<ISIAPISite> site, CComPtr<ISIAPILeg> leg, std::list<long> possibleDests);

		int SetLaneDisciplines(CComPtr<ISIAPILaneApproach> laneAppr, std::list<long> destinations);

		int CovertODToLocalTurn(long origin, long destination, long *localTurn);

	private:
		CComPtr<ISIAPI> pISIAPI;

	};
}
