#include "stdafx.h"
#include "tchar.h"

#include "CppSIAPI.h"
#include "CppSIAPITest.h"

#include <iostream>
#include <string>
#include <list>
#include <time.h>
#include <ATLComTime.h>

#include <atlcomcli.h>

namespace CppSample
{
	CppSIAPITest::CppSIAPITest()
	{

	}

	CppSIAPITest::CppSIAPITest(CppSIAPI *api)
	{
		pCppSIAPI = api;
		pISIAPI = api->get_SIAPI();
	}

	CppSIAPITest::~CppSIAPITest()
	{
	}

	// A sample to create two Signals Sites and build a Freeway Diamond Interchange Network
	int CppSIAPITest::CreateFreewayDiamondInterchangeNetwork()
	{
		VARIANT_BOOL boolResult = false;
		try
		{
			pISIAPI->CreateAndOpenProject(CComBSTR("C:\\Temp\\APITest.sip8"), CComBSTR("APITest"), &boolResult);

			CComPtr<ISIAPIProject> project;
			pISIAPI->get_Project(&project);

			CComBSTR name;
			project->get_Name(&name);
			wprintf(L"\nProject '%s' is created.\n", name);

			CComPtr<ISIAPISite> site209W = CreateSite209W(project);
			CComPtr<ISIAPISite> site209E = CreateSite209E(project);

			CComPtr<ISIAPINetwork> network = CreateNetwork(project);

			if (network == NULL)
				return 1;
			
			VARIANT_BOOL boolResult = false;

			wprintf(L"\nProcessing Network...\n");

			network->Process(&boolResult);
			if (boolResult)
			{
				CComPtr<ISIAPIOutputNetwork> outputNw = NULL;
				network->get_OutputNetwork(&outputNw);

				if (outputNw == NULL)
					throw L"Error - No Network Output Data!";

				DATE generateDate = 0.0;
				outputNw->get_GeneratedTime(&generateDate);

				COleDateTime oleDate = generateDate;

				wprintf(L"Processed network at %02i/%02i/%i %02i:%02i:%02i.",
					oleDate.GetDay(), oleDate.GetMonth(), oleDate.GetYear(),
					oleDate.GetHour(), oleDate.GetMinute(), oleDate.GetSecond());

			}
			else
			{
				CComBSTR errStr = NULL;
				network->get_ProcessingError(&errStr);
				wprintf(L"%s\n", errStr);
			}

			wprintf(L"\nFinish successfully.\n");
		}
		catch (wchar_t* param)
		{
			wprintf(param);
		}
		catch (...)
		{
			wprintf(L"Error!");
		}

		pISIAPI->CloseProject(&boolResult);

		return 0;
	}

	CComPtr<ISIAPISite> CppSIAPITest::CreateSite209W(CComPtr<ISIAPIProject> project)
	{
		VARIANT_BOOL boolResult = false;
		
		CComPtr<ISIAPISite> site;
		project->AddSite(0, 1, &site);

		CComBSTR siteName("209W");
		site->put_Name(siteName);
		site->put_Intersectionid(siteName);

		wprintf(L"Created Signalised Site '%s'.\n", siteName);

		CComPtr<ISIAPIMovementClasses> movClasses;
		site->get_MovementClasses(&movClasses);

		CComPtr<ISIAPIMovementClass> mcLF;
		movClasses->get_Item(7l, &mcLF);
		mcLF->put_Is_included(true);
		mcLF->put_Base_class(1l);
		mcLF->put_Name(CComBSTR("LV to Freeway"));
		mcLF->put_Display_id(CComBSTR("LF"));

		CComPtr<ISIAPIMovementClass> mcHF;
		movClasses->get_Item(8l, &mcHF);
		mcHF->put_Is_included(true);
		mcHF->put_Base_class(2l);
		mcHF->put_Name(CComBSTR("HV to Freeway"));
		mcHF->put_Display_id(CComBSTR("HF"));

		CComPtr<ISIAPILegs> legs;
		CComPtr<ISIAPILeg> leg;
		CComPtr<ISIAPILaneApproachs> laneApprs;
		CComPtr<ISIAPILaneApproach> laneAppr;
		CComPtr<ISIAPILaneApproachMovements> laneApprMovs;
		CComPtr<ISIAPILaneApproachMovement> laneApprMov;
		CComPtr<ISIAPILaneApproachMovementMCs> laneApprMovMCs;
		CComPtr<ISIAPILaneApproachMovementMC> laneApprMovMC;
		CComPtr<ISIAPIIsland> island;
		CComPtr<ISIAPILaneExit> laneExit;
		CComPtr<ISIAPILaneSegment> laneSeg;
		CComPtr<ISIAPILaneMovements> laneMovs;
		CComPtr<ISIAPILaneMovement> laneMov;
		CComPtr<ISIAPILaneMovementMCs> laneMovMCs;
		CComPtr<ISIAPILaneMovementMC> laneMovMC;
		CComPtr<ISIAPIMovement_vehicle_ods> movVehODs;
		CComPtr<ISIAPIMovement_vehicle_od> movVehOD;
		CComPtr<ISIAPIMovement_vehicle_od_mcs> movVehODMCs;
		CComPtr<ISIAPIMovement_vehicle_od_mc> movVehODMC;
		CComPtr<ISIAPIMovement_peds> movPeds;
		CComPtr<ISIAPIMovement_ped> movPed;
		long position = 1;

		// Add a Leg on SouthEast
		site->AddLeg(1l, &leg);
		leg->put_Name(CComBSTR(L"Freeway Off-Ramp South"));
		leg->put_Approach_distance(200.0f);

		position = 1;

		// Add an Approach Lane
		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Type(2l); // Slip/Bypass (High Angle)
		laneAppr->put_Slip_control_type(1l); // Signals Control Type
		laneAppr->put_Width(3.5f);
		laneAppr->put_Grade(3.0f);

		// Set the Lane Discipline from this Approach Lane to the West (destination: 6) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(6l, &laneApprMov); // To West
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		// Add an Approach Lane and set the Discipline to West
		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Type(2l); // Slip/Bypass (High Angle)
		laneAppr->put_Slip_control_type(1l); // Signals Control Type
		laneAppr->put_Width(3.5f);
		laneAppr->put_Grade(3.0f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(6l, &laneApprMov); // To West
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		// Add an Approach Lane and set the Discipline to East
		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);
		laneAppr->put_Grade(3.0f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(2l, &laneApprMov);  // To East
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		// Add a Leg on East
		site->AddLeg(2l, &leg);
		leg->put_Name(CComBSTR(L"Westbound Internal"));
		leg->put_Approach_distance(70.0f);

		position = 1;

		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(6l, &laneApprMov); // To West
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(6l, &laneApprMov); // To West
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(6l, &laneApprMov); // To West
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(3l, &laneApprMov); // To NorthEast
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(7l, &laneApprMovMC); // set User Class 7 Discipline to NorthEast Leg
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(8l, &laneApprMovMC); // set User Class 8 Discipline to NorthEast Leg
		laneApprMovMC->put_Exists(true);

		// Add an Island
		leg->InsertIsland(position++, &island);
		island->put_Width(3.5f);

		// Add four Exit Lanes
		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);

		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);

		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);

		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);


		// Add a Leg on NorthEast
		site->AddLeg(3l, &leg);
		leg->put_Name(CComBSTR(L"Freeway On-Ramp North"));
		leg->put_Approach_distance(200.0f);
		leg->put_Exit_distance_user(true);
		leg->put_Exit_distance(200.0f);

		position = 1;

		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);
		laneExit->put_Grade(-3.0f);

		// Add a Leg on West
		site->AddLeg(6l, &leg);
		leg->put_Name(CComBSTR(L"Eastbound External"));
		leg->put_Approach_distance(500.0f);

		position = 1;

		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Configuration(2l); // Short Lane
		laneAppr->put_Type(2l); // Slip/Bypass (High Angle)
		laneAppr->put_Slip_control_type(2l); // Giveway/Yield Control Type
		laneAppr->put_Width(3.5f);

		laneAppr->put_Length(90.0f);
		laneAppr->get_Segment1(&laneSeg);
		laneSeg->put_Overflow_merge_lane_num_1(2l); // Overflow to Lane 2

		laneAppr->put_Is_sliplane_excluded_from_signal_analysis(true);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(3l, &laneApprMov); // To NorthEast
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(2l, &laneApprMov); // To East
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(2l, &laneApprMov); // To East
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(2l, &laneApprMov); // To East
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Configuration(2l); // Short Lane
		laneAppr->put_Width(3.5f);
		laneAppr->put_Length(90.0f);
		laneAppr->get_Segment1(&laneSeg);
		laneSeg->put_Overflow_merge_lane_num_1(4l); // Overflow to Lane 4


		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(2l, &laneApprMov); // To East
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(7l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(8l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		leg->InsertIsland(position++, &island);
		island->put_Width(5.0f);
		island->put_Width_back(8.5f);

		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);

		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);

		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);

		wprintf(L"Updating Geometry Data...\n");

		// Call the method UpdateGeometryData() after Leg(s) and Lane(s) are added or removed.
		site->UpdateGeometryData(&boolResult);

		wprintf(L"Updated Geometry Data.\n");

		// Change the Lane Movement Flow Proportions from SouthEast Approach Lane 3 to East Exit Lane 1, 2, 3, 4
		// Lane Movement data can be accessed only after site.UpdateGeometryData() is called.
		site->get_Legs(&legs);
		legs->get_Item(1l, &leg);
		leg->get_LaneApproachs(&laneApprs);
		laneApprs->get_Item(3l, &laneAppr);
		laneAppr->get_LaneMovements(&laneMovs);

		laneMovs->get_Item(2l, 1l, &laneMov);

		laneMov->get_LaneMovementMCs(&laneMovMCs);
		laneMovMCs->get_Item(1l, &laneMovMC);
		laneMovMC->put_Flow_proportion(33.0f);
		laneMovMCs->get_Item(2l, &laneMovMC);
		laneMovMC->put_Flow_proportion(33.0f);

		laneMovs->get_Item(2l, 2l, &laneMov);

		laneMov->get_LaneMovementMCs(&laneMovMCs);
		laneMovMCs->get_Item(1l, &laneMovMC);
		laneMovMC->put_Flow_proportion(34.0f);
		laneMovMCs->get_Item(2l, &laneMovMC);
		laneMovMC->put_Flow_proportion(34.0f);

		laneMovs->get_Item(2l, 3l, &laneMov);

		laneMov->get_LaneMovementMCs(&laneMovMCs);
		laneMovMCs->get_Item(1l, &laneMovMC);
		laneMovMC->put_Flow_proportion(33.0f);
		laneMovMCs->get_Item(2l, &laneMovMC);
		laneMovMC->put_Flow_proportion(33.0f);

		laneMovs->get_Item(2l, 4l, &laneMov);

		// by default, this Lane Moveemnt Flow Proportion is 100%, change it to 0.
		laneMov->get_LaneMovementMCs(&laneMovMCs);
		laneMovMCs->get_Item(1l, &laneMovMC);
		laneMovMC->put_Flow_proportion(0.0f);
		laneMovMCs->get_Item(2l, &laneMovMC);
		laneMovMC->put_Flow_proportion(0.0f);

		// Set Vehicle Origin-Destination (OD) Movements data, e.g. Volumes, Approach Cruise Speed and Exit Cruise Speed 
		site->get_MovementVehicleODs(&movVehODs);
		movVehODs->get_Item(1l, 6l, &movVehOD);   // Vehicle OD Movement SouthEast to West
		movVehOD->put_Display_od_id(CComBSTR("1W"));

		movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
		movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
		movVehODMC->put_Volume(830.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
		movVehODMC->put_Volume(25.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODs->get_Item(1l, 3l, &movVehOD);   // Vehicle OD Movement SouthEast to NorthEast
		movVehOD->put_Exists(false);  // set it not existing

		movVehODs->get_Item(1l, 2l, &movVehOD);   // Vehicle OD Movement SouthEast to East
		movVehOD->put_Display_od_id(CComBSTR("2W"));

		movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
		movVehODMCs->get_Item(1l, &movVehODMC);
		movVehODMC->put_Volume(39.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(40.0f);

		movVehODMCs->get_Item(2l, &movVehODMC);
		movVehODMC->put_Volume(3.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(40.0f);

		movVehODs->get_Item(2l, 6l, &movVehOD);   // Vehicle OD Movement East to West
		movVehOD->put_Display_od_id(CComBSTR("3W"));

		movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
		movVehODMCs->get_Item(1l, &movVehODMC);
		movVehODMC->put_Volume(1134.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(40.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODMCs->get_Item(2l, &movVehODMC);
		movVehODMC->put_Volume(36.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(40.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODs->get_Item(2l, 3l, &movVehOD);   // Vehicle OD Movement East to NorthEast
		movVehOD->put_Display_od_id(CComBSTR("4W"));

		movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
		movVehODMCs->get_Item(7l, &movVehODMC); // User Class 7 Movement
		movVehODMC->put_Volume(215.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(40.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODMCs->get_Item(8l, &movVehODMC);  // User Class 8 Movement
		movVehODMC->put_Volume(7.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(40.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODs->get_Item(6l, 3l, &movVehOD);   // Vehicle OD Movement West to NorthEast
		movVehOD->put_Display_od_id(CComBSTR("5W"));

		movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
		movVehODMCs->get_Item(1l, &movVehODMC);
		movVehODMC->put_Volume(280.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODMCs->get_Item(2l, &movVehODMC);
		movVehODMC->put_Volume(9.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODs->get_Item(6l, 2l, &movVehOD);   // Vehicle OD Movement West to East
		movVehOD->put_Display_od_id(CComBSTR("6W"));

		movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
		movVehODMCs->get_Item(1l, &movVehODMC);
		movVehODMC->put_Volume(561.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(40.0f);

		movVehODMCs->get_Item(2l, &movVehODMC);
		movVehODMC->put_Volume(15.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(40.0f);

		movVehODMCs->get_Item(7l, &movVehODMC);
		movVehODMC->put_Volume(306.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(40.0f);

		movVehODMCs->get_Item(8l, &movVehODMC);
		movVehODMC->put_Volume(10.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(40.0f);

		// Turn off all Pedestrian Movements
		site->get_MovementPeds(&movPeds);
		movPeds->get_Item(1l, 1l, 0l, &movPed);
		movPed->put_Exists(false);
		movPeds->get_Item(1l, 2l, 0l, &movPed);
		movPed->put_Exists(false);
		movPeds->get_Item(1l, 3l, 0l, &movPed);
		movPed->put_Exists(false);
		movPeds->get_Item(1l, 6l, 0l, &movPed);
		movPed->put_Exists(false);

		// Set the Phasing and Timing data
		// Add a new Sequence named "Diamond Interchange"
		CComPtr<ISIAPISequence> sequence;
		CComPtr<ISIAPIPhases> phases;
		CComPtr<ISIAPIPhase> phase;
		CComPtr<ISIAPIPhasemovement_vehicles> phaseMovVehs;
		CComPtr<ISIAPIPhasemovement_vehicle> phaseMovVeh;

		site->AddSequence(CComBSTR("Diamond Interchange"), &sequence);
		sequence->put_Is_selected(true);

		// Add a new Phase "A" to the Sequence
		sequence->AddPhase(CComBSTR("A"), &phase);
		phase->put_IsReferencePhase(true);  // set it as the Referenced Phase
		phase->get_Phasemovement_vehicles(&phaseMovVehs);

		phaseMovVehs->get_Item(2l, 6l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(2l, 6l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		phaseMovVehs->get_Item(6l, 3l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 3l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		phaseMovVehs->get_Item(6l, 2l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 2l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 2l, 7l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 2l, 8l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		sequence->AddPhase(CComBSTR("C"), &phase);
		phase->get_Phasemovement_vehicles(&phaseMovVehs);

		phaseMovVehs->get_Item(2l, 6l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(2l, 6l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		phaseMovVehs->get_Item(2l, 3l, 7l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(2l, 3l, 8l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		sequence->AddPhase(CComBSTR("D"), &phase);
		phase->get_Phasemovement_vehicles(&phaseMovVehs);

		phaseMovVehs->get_Item(1l, 6l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(1l, 6l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		phaseMovVehs->get_Item(1l, 2l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(1l, 2l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);


		pISIAPI->SubmitDataChanges(&boolResult);



		//CComBSTR idstr;
		//CComBSTR siteName2;
		//site->get_Name(&siteName2);
		//site->get_Intersectionid(&idstr);
		//wprintf(L"Created Signalised Site '%s' '%s'.\n", siteName2, idstr);

		return site;

	}

	CComPtr<ISIAPISite> CppSIAPITest::CreateSite209E(CComPtr<ISIAPIProject> project)
	{
		VARIANT_BOOL boolResult = false;

		CComPtr<ISIAPISite> site;
		project->AddSite(0, 1, &site);

		CComBSTR siteName("209E");
		site->put_Name(siteName);
		site->put_Intersectionid(siteName);

		wprintf(L"Created Signalised Site '%s'.\n", siteName);

		CComPtr<ISIAPIMovementClasses> movClasses;
		site->get_MovementClasses(&movClasses);

		CComPtr<ISIAPIMovementClass> mcLF;
		movClasses->get_Item(7l, &mcLF);
		mcLF->put_Is_included(true);
		mcLF->put_Base_class(1l);
		mcLF->put_Name(CComBSTR("LV to Freeway"));
		mcLF->put_Display_id(CComBSTR("LF"));

		CComPtr<ISIAPIMovementClass> mcHF;
		movClasses->get_Item(8l, &mcHF);
		mcHF->put_Is_included(true);
		mcHF->put_Base_class(2l);
		mcHF->put_Name(CComBSTR("HV to Freeway"));
		mcHF->put_Display_id(CComBSTR("HF"));

		CComPtr<ISIAPILegs> legs;
		CComPtr<ISIAPILeg> leg;
		CComPtr<ISIAPILaneApproachs> laneApprs;
		CComPtr<ISIAPILaneApproach> laneAppr;
		CComPtr<ISIAPILaneApproachMovements> laneApprMovs;
		CComPtr<ISIAPILaneApproachMovement> laneApprMov;
		CComPtr<ISIAPILaneApproachMovementMCs> laneApprMovMCs;
		CComPtr<ISIAPILaneApproachMovementMC> laneApprMovMC;
		CComPtr<ISIAPIIsland> island;
		CComPtr<ISIAPILaneExit> laneExit;
		CComPtr<ISIAPILaneSegment> laneSeg;
		CComPtr<ISIAPILaneMovements> laneMovs;
		CComPtr<ISIAPILaneMovement> laneMov;
		CComPtr<ISIAPILaneMovementMCs> laneMovMCs;
		CComPtr<ISIAPILaneMovementMC> laneMovMC;
		CComPtr<ISIAPIMovement_vehicle_ods> movVehODs;
		CComPtr<ISIAPIMovement_vehicle_od> movVehOD;
		CComPtr<ISIAPIMovement_vehicle_od_mcs> movVehODMCs;
		CComPtr<ISIAPIMovement_vehicle_od_mc> movVehODMC;
		CComPtr<ISIAPIMovement_peds> movPeds;
		CComPtr<ISIAPIMovement_ped> movPed;
		long position = 1;

		// Add a Leg on East
		site->AddLeg(2l, &leg);
		leg->put_Name(CComBSTR(L"Westbound External"));
		leg->put_Approach_distance(500.0f);

		position = 1;

		// Add an Approach Lane
		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Configuration(2l); // Short Lane
		laneAppr->put_Type(2l); // Slip/Bypass (High Angle)
		laneAppr->put_Slip_control_type(2l); // Giveway/Yield Control Type
		laneAppr->put_Width(3.5f);

		laneAppr->put_Length(90.0f);
		laneAppr->get_Segment1(&laneSeg);
		laneSeg->put_Overflow_merge_lane_num_1(2l); // Overflow to Lane 2

		laneAppr->put_Is_sliplane_excluded_from_signal_analysis(true);

		// Set the Lane Discipline from this Approach Lane to the SouthWest (destination: 7) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(7l, &laneApprMov); // To SouthWest
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		// Add an Approach Lane
		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(6l, &laneApprMov); // To West
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		// Add an Approach Lane
		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(6l, &laneApprMov); // To West
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		// Add an Approach Lane
		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(6l, &laneApprMov); // To West
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		// Add an Approach Lane
		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Configuration(2l); // Short Lane
		laneAppr->put_Width(3.5f);

		laneAppr->put_Length(90.0f);
		laneAppr->get_Segment1(&laneSeg);
		laneSeg->put_Overflow_merge_lane_num_1(4l); // Overflow to Lane 4

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(6l, &laneApprMov); // To West
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(7l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(8l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		// Add an Island
		leg->InsertIsland(position++, &island);
		island->put_Width(5.0f);
		island->put_Width_back(8.5f);

		// Add three Exit Lanes
		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);

		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);

		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);

		// Add a Leg on NorthWest
		site->AddLeg(5l, &leg);
		leg->put_Name(CComBSTR(L"Freeway Off-Ramp North"));
		leg->put_Approach_distance(200.0f);

		position = 1;

		// Add an Approach Lane
		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Type(2l); // Slip/Bypass (High Angle)
		laneAppr->put_Slip_control_type(2l); // Giveway/Yield Control Type
		laneAppr->put_Width(3.5f);
		laneAppr->put_Grade(3.0f);

		laneAppr->put_Is_sliplane_excluded_from_signal_analysis(true);

		// Set the Lane Discipline from this Approach Lane to the East (destination: 2) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(2l, &laneApprMov); // To East
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		// Add an Approach Lane and set the Discipline to West
		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);
		laneAppr->put_Grade(3.0f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(6l, &laneApprMov); // To West
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);


		// Add a Leg on West
		site->AddLeg(6l, &leg);
		leg->put_Name(CComBSTR(L"Eastbound Internal"));
		leg->put_Approach_distance(70.0f);

		position = 1;

		// Add an Approach Lane
		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(2l, &laneApprMov); // To East
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		// Add an Approach Lane
		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(2l, &laneApprMov); // To East
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		// Add an Approach Lane
		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(2l, &laneApprMov); // To East
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(1l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(2l, &laneApprMovMC);
		laneApprMovMC->put_Exists(true);

		// Add an Approach Lane
		leg->InsertApproachLane(position++, &laneAppr);
		laneAppr->put_Width(3.5f);

		laneAppr->get_LaneApproachMovements(&laneApprMovs);
		laneApprMovs->get_Item(7l, &laneApprMov); // To SouthWest
		laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
		laneApprMovMCs->get_Item(7l, &laneApprMovMC); // set User Class 7 Discipline to NorthEast Leg
		laneApprMovMC->put_Exists(true);
		laneApprMovMCs->get_Item(8l, &laneApprMovMC); // set User Class 8 Discipline to NorthEast Leg
		laneApprMovMC->put_Exists(true);

		// Add an Island
		leg->InsertIsland(position++, &island);
		island->put_Width(1.5f);
		island->put_Width_back(1.5f);

		// Add four Exit Lanes
		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);

		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);

		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);

		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);

		// Add a Leg on SouthWest
		site->AddLeg(7l, &leg);
		leg->put_Name(CComBSTR(L"Freeway On-Ramp South"));
		leg->put_Approach_distance(200.0f);
		leg->put_Exit_distance_user(true);
		leg->put_Exit_distance(200.0f);

		position = 1;

		leg->InsertExitLane(position++, &laneExit);
		laneExit->put_Width(3.5f);
		laneExit->put_Grade(-3.0f);

		wprintf(L"Updating Geometry Data...\n");

		// Call the method UpdateGeometryData() after Leg(s) and Lane(s) are added or removed.
		site->UpdateGeometryData(&boolResult);

		wprintf(L"Updated Geometry Data.\n");

		// Set the Lane Movement Flow Proportions from NorthWest Approach Lane 2 to West Exit Lane 1, 2, 3
		// Lane Movement data can be accessed only after site.UpdateGeometryData() is called.
		site->get_Legs(&legs);
		legs->get_Item(5l, &leg);
		leg->get_LaneApproachs(&laneApprs);
		laneApprs->get_Item(2l, &laneAppr);
		laneAppr->get_LaneMovements(&laneMovs);

		laneMovs->get_Item(6l, 1l, &laneMov);
		laneMov->get_LaneMovementMCs(&laneMovMCs);
		laneMovMCs->get_Item(1l, &laneMovMC);
		laneMovMC->put_Flow_proportion(33.0f);
		laneMovMCs->get_Item(2l, &laneMovMC);
		laneMovMC->put_Flow_proportion(33.0f);

		laneMovs->get_Item(6l, 2l, &laneMov);
		laneMov->get_LaneMovementMCs(&laneMovMCs);
		laneMovMCs->get_Item(1l, &laneMovMC);
		laneMovMC->put_Flow_proportion(34.0f);
		laneMovMCs->get_Item(2l, &laneMovMC);
		laneMovMC->put_Flow_proportion(34.0f);

		laneMovs->get_Item(6l, 3l, &laneMov);
		laneMov->get_LaneMovementMCs(&laneMovMCs);
		laneMovMCs->get_Item(1l, &laneMovMC);
		laneMovMC->put_Flow_proportion(33.0f);
		laneMovMCs->get_Item(2l, &laneMovMC);
		laneMovMC->put_Flow_proportion(33.0f);

		laneMovs->get_Item(6l, 4l, &laneMov); // by default, this Lane Moveemnt Flow Proportion is 100%, change it to 0.
		laneMov->get_LaneMovementMCs(&laneMovMCs);
		laneMovMCs->get_Item(1l, &laneMovMC);
		laneMovMC->put_Flow_proportion(0.0f);
		laneMovMCs->get_Item(2l, &laneMovMC);
		laneMovMC->put_Flow_proportion(0.0f);

		// Set Vehicle Origin-Destination (OD) Movements data, e.g. Volumes, Approach Cruise Speed and Exit Cruise Speed 
		site->get_MovementVehicleODs(&movVehODs);
		movVehODs->get_Item(5l, 2l, &movVehOD);   // Vehicle OD Movement NorthWest to East
		movVehOD->put_Display_od_id(CComBSTR("1E"));
		movVehOD->get_MovementVehicleODMCs(&movVehODMCs);

		movVehODMCs->get_Item(1l, &movVehODMC);   // Light Vehicle Movement
		movVehODMC->put_Volume(186.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODMCs->get_Item(2l, &movVehODMC);   // Heavy Vehicle Movement
		movVehODMC->put_Volume(8.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODs->get_Item(5l, 7l, &movVehOD);   // Vehicle OD Movement NorthWest to SouthWest
		movVehOD->put_Exists(false); // set it not existing

		movVehODs->get_Item(5l, 6l, &movVehOD);   // Vehicle OD Movement NorthWest to West
		movVehOD->put_Display_od_id(CComBSTR("2E"));
		movVehOD->get_MovementVehicleODMCs(&movVehODMCs);

		movVehODMCs->get_Item(1l, &movVehODMC);   // Light Vehicle Movement
		movVehODMC->put_Volume(192.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(40.0f);

		movVehODMCs->get_Item(2l, &movVehODMC);   // Heavy Vehicle Movement
		movVehODMC->put_Volume(6.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(40.0f);

		movVehODs->get_Item(6l, 2l, &movVehOD);   // Vehicle OD Movement West to East
		movVehOD->put_Display_od_id(CComBSTR("3E"));
		movVehOD->get_MovementVehicleODMCs(&movVehODMCs);

		movVehODMCs->get_Item(1l, &movVehODMC);   // Light Vehicle Movement
		movVehODMC->put_Volume(600.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(40.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODMCs->get_Item(2l, &movVehODMC);   // Heavy Vehicle Movement
		movVehODMC->put_Volume(18.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(40.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODs->get_Item(6l, 7l, &movVehOD);   // Vehicle OD Movement West to SouthWest
		movVehOD->put_Display_od_id(CComBSTR("4E"));
		movVehOD->get_MovementVehicleODMCs(&movVehODMCs);

		movVehODMCs->get_Item(7l, &movVehODMC);   // User Class 7 Movement
		movVehODMC->put_Volume(306.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(40.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODMCs->get_Item(8l, &movVehODMC);   // User Class 8 Movement
		movVehODMC->put_Volume(10.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(40.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODs->get_Item(2l, 7l, &movVehOD);   // Vehicle OD Movement East to SouthWest
		movVehOD->put_Display_od_id(CComBSTR("5E"));
		movVehOD->get_MovementVehicleODMCs(&movVehODMCs);

		movVehODMCs->get_Item(1l, &movVehODMC);   // Light Vehicle Movement
		movVehODMC->put_Volume(120.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODMCs->get_Item(2l, &movVehODMC);   // Heavy Vehicle Movement
		movVehODMC->put_Volume(4.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(70.0f);

		movVehODs->get_Item(2l, 6l, &movVehOD);   // Vehicle OD Movement East to West
		movVehOD->put_Display_od_id(CComBSTR("6E"));
		movVehOD->get_MovementVehicleODMCs(&movVehODMCs);

		movVehODMCs->get_Item(1l, &movVehODMC);   // Light Vehicle Movement
		movVehODMC->put_Volume(942.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(40.0f);

		movVehODMCs->get_Item(2l, &movVehODMC);   // Heavy Vehicle Movement
		movVehODMC->put_Volume(30.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(40.0f);

		movVehODMCs->get_Item(7l, &movVehODMC);   // User Class 7 Movement
		movVehODMC->put_Volume(215.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(40.0f);

		movVehODMCs->get_Item(8l, &movVehODMC);   // User Class 8 Movement
		movVehODMC->put_Volume(7.0f);
		movVehODMC->put_Peak_flow_factor(100.0f);
		movVehODMC->put_Approach_speed(70.0f);
		movVehODMC->put_Exit_speed(40.0f);

		// Turn off all Pedestrian Movements
		site->get_MovementPeds(&movPeds);
		movPeds->get_Item(1l, 2l, 0l, &movPed);
		movPed->put_Exists(false);
		movPeds->get_Item(1l, 5l, 0l, &movPed);
		movPed->put_Exists(false);
		movPeds->get_Item(1l, 6l, 0l, &movPed);
		movPed->put_Exists(false);
		movPeds->get_Item(1l, 7l, 0l, &movPed);
		movPed->put_Exists(false);

		// Set the Phasing and Timing data
		// Add a new Sequence named "Diamond Interchange"
		CComPtr<ISIAPISequence> sequence;
		CComPtr<ISIAPIPhases> phases;
		CComPtr<ISIAPIPhase> phase;
		CComPtr<ISIAPIPhasemovement_vehicles> phaseMovVehs;
		CComPtr<ISIAPIPhasemovement_vehicle> phaseMovVeh;

		site->AddSequence(CComBSTR("Diamond Interchange"), &sequence);
		sequence->put_Is_selected(true);

		// Add a new Phase "A" to the Sequence
		sequence->AddPhase(CComBSTR("A"), &phase);
		phase->put_IsReferencePhase(true);  // set it as the Referenced Phase
		phase->get_Phasemovement_vehicles(&phaseMovVehs);

		phaseMovVehs->get_Item(2l, 6l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(2l, 6l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(2l, 6l, 7l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(2l, 6l, 8l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 2l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 2l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		sequence->AddPhase(CComBSTR("C"), &phase);
		phase->get_Phasemovement_vehicles(&phaseMovVehs);

		phaseMovVehs->get_Item(6l, 2l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 2l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 7l, 7l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 7l, 8l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		sequence->AddPhase(CComBSTR("D"), &phase);
		phase->get_Phasemovement_vehicles(&phaseMovVehs);

		phaseMovVehs->get_Item(5l, 6l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(5l, 6l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		pISIAPI->SubmitDataChanges(&boolResult);

		return site;
	}

	CComPtr<ISIAPINetwork> CppSIAPITest::CreateNetwork(CComPtr<ISIAPIProject> project)
	{
		CComPtr<ISIAPISites> sites = NULL;
		CComPtr<ISIAPISite> site209W = NULL;
		CComPtr<ISIAPISite> site209E = NULL;

		CComPtr<ISIAPINetwork> network = NULL;
		CComPtr<ISIAPINetworkSite> nwSite209W = NULL;
		CComPtr<ISIAPINetworkSite> nwSite209E = NULL;
		CComPtr<ISIAPINetworkLegConnection> nwLegConn = NULL;

		CComPtr<ISIAPINetworkCCG> ccg = NULL;
		CComPtr<ISIAPINetworkCCGSequence> ccgSeq = NULL;
		CComPtr<ISIAPINetworkCCGPhase> ccgPhase = NULL;
		CComPtr<ISIAPIPhasemovement_vehicles> phaseMovVehs = NULL;
		CComPtr<ISIAPIPhasemovement_vehicle> phaseMovVeh = NULL;

		CComPtr<ISIAPIRoute> route = NULL;
		CComPtr<ISIAPIRouteNwSite> routeNwSite = NULL;

		VARIANT_BOOL boolResult;

		project->AddNetwork(1, &network);

		network->put_Name(CComBSTR("Freeway Diamond Interchange"));
		network->put_NetworkID(CComBSTR("209"));

		CComBSTR networkName = NULL;
		network->get_Name(&networkName);

		network->put_DesiredSpeedMethod(0);  // User-input
		network->put_DesiredSpeed(70.0f);

		project->get_Sites(&sites);
		sites->get_Item(CComBSTR("209W"), &site209W);
		sites->get_Item(CComBSTR("209E"), &site209E);

		network->AddNetworkSite(site209W, 5200.0f, 5100.0f, &nwSite209W);
		network->AddNetworkSite(site209E, 5470.0f, 5100.0f, &nwSite209E);

		network->AddNetworkLegConnection(nwSite209W, 2l, nwSite209E, 6l, &nwLegConn);

		wprintf(L"Created Network '%s'.\n", networkName);

		network->AddNetworkCCG(&ccg);
		ccg->put_Name(CComBSTR("Fwy Int"));
		ccg->put_NetworkCCGID(CComBSTR("209"));

		ccg->AddNetworkSite(nwSite209W, &boolResult);
		ccg->AddNetworkSite(nwSite209E, &boolResult);

		ccg->AddCCGSequence(CComBSTR("CCG Phasing"), &ccgSeq);
		ccgSeq->put_Is_selected(true);
		ccgSeq->put_Cycle_time_option(2); // User-Given Cycle Time
		ccgSeq->put_Usergiven_cycle_time(90);
		ccgSeq->put_Green_split_priority_option(1l);

		ccgSeq->AddCCGPhase(CComBSTR("A"), &ccgPhase);
		ccgPhase->put_IsReferencePhase(true);

		ccgPhase->GetPhasemovementVehiclesByNetworkSite(nwSite209W, &phaseMovVehs);
		phaseMovVehs->get_Item(2l, 6l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(2l, 6l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		phaseMovVehs->get_Item(6l, 2l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 2l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 2l, 7l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 2l, 8l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		ccgPhase->GetPhasemovementVehiclesByNetworkSite(nwSite209E, &phaseMovVehs);
		phaseMovVehs->get_Item(2l, 6l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(2l, 6l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(2l, 6l, 7l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(2l, 6l, 8l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		phaseMovVehs->get_Item(6l, 2l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 2l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		ccgSeq->AddCCGPhase(CComBSTR("C"), &ccgPhase);

		ccgPhase->GetPhasemovementVehiclesByNetworkSite(nwSite209W, &phaseMovVehs);
		phaseMovVehs->get_Item(2l, 6l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(2l, 6l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		phaseMovVehs->get_Item(2l, 3l, 7l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(2l, 3l, 8l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		ccgPhase->GetPhasemovementVehiclesByNetworkSite(nwSite209E, &phaseMovVehs);
		phaseMovVehs->get_Item(6l, 2l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 2l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 7l, 7l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(6l, 7l, 8l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		ccgSeq->AddCCGPhase(CComBSTR("D"), &ccgPhase);

		ccgPhase->GetPhasemovementVehiclesByNetworkSite(nwSite209W, &phaseMovVehs);
		phaseMovVehs->get_Item(1l, 6l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(1l, 6l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(1l, 2l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(1l, 2l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		ccgPhase->GetPhasemovementVehiclesByNetworkSite(nwSite209E, &phaseMovVehs);
		phaseMovVehs->get_Item(5l, 6l, 1l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);
		phaseMovVehs->get_Item(5l, 6l, 2l, &phaseMovVeh);
		phaseMovVeh->put_Running(true);

		wprintf(L"Added CCG and its Phasing data.\n");

		network->AddRoute(&route);
		route->AddRouteNwSite(nwSite209W, 6l, 2l, &routeNwSite);
		route->AddRouteNwSite(nwSite209E, 6l, 2l, &routeNwSite);

		route->CheckIsValidForSignalOffsetCalc(&boolResult);

		route->put_IsIncludedInOutputNetworkByRoutes(true);

		pISIAPI->SubmitDataChanges(&boolResult);

		return network;
	}

	// A sample to read Lane Disciplines data
	int CppSIAPITest::ReadLaneDisciplines()
	{
		VARIANT_BOOL boolResult = false;

		try
		{
			pISIAPI->OpenProject(CComBSTR("C:\\Temp\\APITest.sip8"), &boolResult);

			CComPtr<ISIAPIProject> project = NULL;
			pISIAPI->get_Project(&project);

			CComPtr<ISIAPISites> sites = NULL;
			project->get_Sites(&sites);

			CComPtr<ISIAPISite> site = NULL;
			CComBSTR name = NULL;

			long siteCount, siteIndex, legCount, origin, laneApprCount, laneno;
			long mcClass, dest;

			CComPtr<ISIAPILegs> legs = NULL;
			CComPtr<ISIAPILeg> leg = NULL;
			CComPtr<ISIAPILaneApproachs> laneApprs = NULL;
			CComPtr<ISIAPILaneApproach> laneAppr = NULL;

			sites->get_Count(&siteCount);
			for (siteIndex = 0; siteIndex < siteCount; siteIndex++)
			{
				sites->get_Item_2(siteIndex, &site);
				if (site != NULL)
				{
					site->get_Name(&name);

					wprintf(L"Reading Site '%s':\n", name);

					bool includedMCs[MAXMcClass];
					pCppSIAPI->ReadIncludedMCs(site, includedMCs);

					//wprintf(L"Movement Classes:\n");
					//for (mcClass = 1; mcClass <= MAXMcClass; mcClass++)
					//	wprintf(L"Movement Class [%d] included = %d\n", mcClass, includedMCs[mcClass - 1]);

					bool existingVehODs[MAXOrientation][MAXOrientation];
					pCppSIAPI->ReadExistingVehODs(site, existingVehODs);

					//wprintf(L"\nMovement Veh ODs:\n");
					//for (origin = 0; origin < MAXOrientation; origin++)
					//	for (dest = 0; dest < MAXOrientation; dest++)
					//		wprintf(L"Movement Veh OD [%d][%d] existing = %d\n", origin, dest, existingVehODs[origin][dest]);

					site->get_Legs(&legs);
					legs->get_Count(&legCount);
					for (origin = 0; origin < MAXOrientation; origin++)
					{
						legs->get_Item(origin, &leg);
						if (leg != NULL)
						{
							leg->get_LaneApproachs(&laneApprs);
							laneApprs->get_Count(&laneApprCount);
							for (laneno = 1; laneno <= laneApprCount; laneno++)
							{
								laneApprs->get_Item(laneno, &laneAppr);
								if (laneAppr != NULL)
								{
									wprintf(L"\nLane Disciplines on Leg %d, Approach Lane %d:\n", origin, laneno);
									bool laneApprMovMCExists[MAXOrientation][MAXMcClass];
									pCppSIAPI->ReadLaneApprLaneDisciplines(laneAppr, origin, existingVehODs, includedMCs, laneApprMovMCExists);
									for (dest = 0; dest < MAXOrientation; dest++)
									{
										for (mcClass = 1; mcClass <= MAXMcClass; mcClass++)
										{
											if (laneApprMovMCExists[dest][mcClass - 1])
												wprintf(L"Lane Approach Mov MC [%d][%d] exists\n", dest, mcClass);
										}
									}
								}
							}
						}
					}

					wprintf(L"\n");
				}
			}
		}
		catch (...)
		{
			//CString str(e->Message);
			wprintf(L"Error!");
		}

		pISIAPI->CloseProject(&boolResult);

		return 0;
	}

	// A sample to read Priorities data
	int CppSIAPITest::ReadPrioritiesData()
	{
		VARIANT_BOOL boolResult = false;
		
		pISIAPI->OpenProject(CComBSTR("C:\\Temp\\APITest.sip8"), &boolResult);

		CComPtr<ISIAPIProject> project = NULL;
		pISIAPI->get_Project(&project);

		CComPtr<ISIAPISites> sites = NULL;
		project->get_Sites(&sites);

		CComPtr<ISIAPISite> site = NULL;

		sites->get_Item_2(0, &site);

		if (site == NULL)
			return 1;

		CComBSTR name = NULL;
		site->get_Name(&name);

		wprintf(L"Reading Site '%s':\n", name);

		bool existingVehODs[MAXOrientation][MAXOrientation];
		pCppSIAPI->ReadExistingVehODs(site, existingVehODs);

		bool existingPeds[MAXOrientation][MAXPedCode];
		bool existingPedDiagonal = false;
		pCppSIAPI->ReadExistingPeds(site, &existingPedDiagonal, existingPeds);

		bool opposingVehData[MAXOrientation][MAXOrientation][MAXOrientation][MAXOrientation];
		pCppSIAPI->ReadOpposingVehData(site, existingVehODs, opposingVehData);

		long origin, dest, opposingOrigin, opposingDest;
		for (origin = 0; origin < MAXOrientation; origin++)
		{
			for (dest = 0; dest < MAXOrientation; dest++)
			{
				if (existingVehODs[origin][dest])
				{
					wprintf(L"\nVeh Mov (%d, %d) is opposed by:\n", origin, dest);
					bool opposed = false;

					for (opposingOrigin = 0; opposingOrigin < MAXOrientation; opposingOrigin++)
					{
						for (opposingDest = 0; opposingDest < MAXOrientation; opposingDest++)
						{
							if (opposingVehData[origin][dest][opposingOrigin][opposingDest])
							{
								opposed = true;
								wprintf(L"Veh Mov (%d, %d)\n", opposingOrigin, opposingDest);
							}
						}
					}

					if (!opposed)
						wprintf(L"None of Veh\n");
				}
			}
		}

		bool opposingPedData[MAXOrientation][MAXOrientation][MAXOrientation][MAXPedCode];
		pCppSIAPI->ReadOpposingPedData(site, existingVehODs, existingPeds, opposingPedData);
		
		long opposingPedOrigin, opposingPedCode, opposingPedType, opposingPedStage;
		for (origin = 0; origin < MAXOrientation; origin++)
		{
			for (dest = 0; dest < MAXOrientation; dest++)
			{
				if (existingVehODs[origin][dest])
				{
					wprintf(L"\nVeh Mov (%d, %d) is opposed by:\n", origin, dest);
					bool opposed = false;

					for (opposingPedOrigin = 0; opposingPedOrigin < MAXOrientation; opposingPedOrigin++)
					{
						for (opposingPedCode = 0; opposingPedCode < MAXPedCode; opposingPedCode++)
						{
							if (opposingPedData[origin][dest][opposingPedOrigin][opposingPedCode])
							{
								opposed = true;
								if (pCppSIAPI->ConvertPedCodeToTypeStage(opposingPedCode, &opposingPedType, &opposingPedStage) == 0)
									wprintf(L"Ped Mov (Type:%d, Origin:%d, Stage:%d)\n", opposingPedType, opposingPedOrigin, opposingPedStage);
								else
									wprintf(L"Wrong Ped Mov Code: %d\n", opposingPedCode);
							}
						}
					}

					if (!opposed)
						wprintf(L"None of Ped\n");
				}
			}
		}

		return 0;
	}

	// A sample to create a Roundabout Site, set the Lane Disciplines specifically
	int CppSIAPITest::CreateRoundaboutSite()
	{
		VARIANT_BOOL boolResult = false;
		
		try
		{
			pISIAPI->CreateAndOpenProject(CComBSTR("C:\\Temp\\Roundabout.sip8"), CComBSTR("Roundabout"), &boolResult);

			CComPtr<ISIAPIProject> project;
			pISIAPI->get_Project(&project);

			CComBSTR name;
			project->get_Name(&name);
			wprintf(L"\nProject '%s' is created.\n", name);

			CComPtr<ISIAPISite> site;
			project->AddSite(1l, 4l, &site);

			CComBSTR siteName("Roundabout1");
			site->put_Name(siteName);

			wprintf(L"Created Roundabout Site '%s'.\n", siteName);

			CComPtr<ISIAPILegs> legs;
			CComPtr<ISIAPILeg> leg;
			//CComPtr<ISIAPILaneApproachs> laneApprs;
			CComPtr<ISIAPILaneApproach> laneAppr;
			CComPtr<ISIAPILaneApproachMovements> laneApprMovs;
			CComPtr<ISIAPILaneApproachMovement> laneApprMov;
			CComPtr<ISIAPILaneApproachMovementMCs> laneApprMovMCs;
			CComPtr<ISIAPILaneApproachMovementMC> laneApprMovMC;
			CComPtr<ISIAPIIsland> island;
			CComPtr<ISIAPILaneExit> laneExit;
			CComPtr<ISIAPILaneSegment> laneSeg;
			CComPtr<ISIAPILaneMovements> laneMovs;
			CComPtr<ISIAPILaneMovement> laneMov;
			CComPtr<ISIAPILaneMovementMCs> laneMovMCs;
			CComPtr<ISIAPILaneMovementMC> laneMovMC;
			CComPtr<ISIAPIMovement_vehicle_ods> movVehODs;
			CComPtr<ISIAPIMovement_vehicle_od> movVehOD;
			CComPtr<ISIAPIMovement_vehicle_od_mcs> movVehODMCs;
			CComPtr<ISIAPIMovement_vehicle_od_mc> movVehODMC;
			CComPtr<ISIAPIMovement_peds> movPeds;
			CComPtr<ISIAPIMovement_ped> movPed;
			long legItemPosition = 1; // for Leg_item Position

			// Add a Leg on South
			site->AddLeg(0l, &leg);
			leg->put_Name(CComBSTR("Eley Road"));

			// Add two Exit Lanes
			leg->InsertExitLane(legItemPosition++, &laneExit);
			leg->InsertExitLane(legItemPosition++, &laneExit);

			// A Roundabout Splitter Island must be added between Approach Lanes and Exit Lanes on each Two-way Leg on a Roundabout Site.
			// A normal Strip Island can be added (by calling the method leg.InsertIsland()) between two Approach Lanes or between two Exit Lanes if required for the geometry layout.
			leg->InsertIsland_RoundaboutSplitter(legItemPosition++, &island);

			// Add an Approach Lane
			leg->InsertApproachLane(legItemPosition++, &laneAppr);

			laneAppr->get_LaneApproachMovements(&laneApprMovs);

			// Set the Lane Discipline from this Approach Lane to the West (destination: 6) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(6l, &laneApprMov); // to West
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			// Set the Lane Discipline from this Approach Lane to the North (destination: 4) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(4l, &laneApprMov); // to North
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			// Add an Approach Lane
			leg->InsertApproachLane(legItemPosition++, &laneAppr);

			laneAppr->get_LaneApproachMovements(&laneApprMovs);

			// Set the Lane Discipline from this Approach Lane to the North (destination: 4) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(4l, &laneApprMov); // to North
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			// Set the Lane Discipline from this Approach Lane to the East (destination: 2) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(2l, &laneApprMov); // to East
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			// Add a Leg on East
			site->AddLeg(2l, &leg);
			leg->put_Name(CComBSTR("Park Road"));

			legItemPosition = 1; // reset the Leg_item position to 1

			// Add two Exit Lanes
			leg->InsertExitLane(legItemPosition++, &laneExit);
			leg->InsertExitLane(legItemPosition++, &laneExit);

			// Add a Roundabout Splitter Island
			leg->InsertIsland_RoundaboutSplitter(legItemPosition++, &island);

			// Add an Approach Lane
			leg->InsertApproachLane(legItemPosition++, &laneAppr);

			laneAppr->get_LaneApproachMovements(&laneApprMovs);

			// Set the Lane Discipline from this Approach Lane to the South (destination: 0) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(0l, &laneApprMov); // to South
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			// Set the Lane Discipline from this Approach Lane to the West (destination: 6) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(6l, &laneApprMov); // to West
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			// Add an Approach Lane
			leg->InsertApproachLane(legItemPosition++, &laneAppr);

			laneAppr->get_LaneApproachMovements(&laneApprMovs);

			// Set the Lane Discipline from this Approach Lane to the West (destination: 6) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(6l, &laneApprMov); // to West
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			// Set the Lane Discipline from this Approach Lane to the North (destination: 4) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(4l, &laneApprMov); // to North
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			// Add a Leg on North
			site->AddLeg(4l, &leg);
			leg->put_Name(CComBSTR("Eley Road"));

			legItemPosition = 1; // reset the Leg_item position to 1

			// Add two Exit Lanes
			leg->InsertExitLane(legItemPosition++, &laneExit);
			leg->InsertExitLane(legItemPosition++, &laneExit);

			// Add a Roundabout Splitter Island
			leg->InsertIsland_RoundaboutSplitter(legItemPosition++, &island);

			// Add an Approach Lane
			leg->InsertApproachLane(legItemPosition++, &laneAppr);

			laneAppr->get_LaneApproachMovements(&laneApprMovs);

			// Set the Lane Discipline from this Approach Lane to the East (destination: 2) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(2l, &laneApprMov); // to East
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			// Set the Lane Discipline from this Approach Lane to the South (destination: 0) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(0l, &laneApprMov); // to South
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			// Add an Approach Lane
			leg->InsertApproachLane(legItemPosition++, &laneAppr);

			laneAppr->get_LaneApproachMovements(&laneApprMovs);

			// Set the Lane Discipline from this Approach Lane to the South (destination: 0) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(0l, &laneApprMov); // to South
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			// Set the Lane Discipline from this Approach Lane to the West (destination: 6) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(6l, &laneApprMov); // to West
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			// Add a Leg on West
			site->AddLeg(6l, &leg);
			leg->put_Name(CComBSTR("Park Road"));

			legItemPosition = 1; // reset the Leg_item position to 1

			// Add two Exit Lanes
			leg->InsertExitLane(legItemPosition++, &laneExit);
			leg->InsertExitLane(legItemPosition++, &laneExit);

			// Add a Roundabout Splitter Island
			leg->InsertIsland_RoundaboutSplitter(legItemPosition++, &island);

			// Add an Approach Lane
			leg->InsertApproachLane(legItemPosition++, &laneAppr);

			laneAppr->get_LaneApproachMovements(&laneApprMovs);

			// Set the Lane Discipline from this Approach Lane to the North (destination: 4) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(4l, &laneApprMov); // to North
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			// Set the Lane Discipline from this Approach Lane to the East (destination: 2) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(2l, &laneApprMov); // to East
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			// Add an Approach Lane
			leg->InsertApproachLane(legItemPosition++, &laneAppr);

			laneAppr->get_LaneApproachMovements(&laneApprMovs);

			// Set the Lane Discipline from this Approach Lane to the East (destination: 2) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(2l, &laneApprMov); // to East
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			// Set the Lane Discipline from this Approach Lane to the South (destination: 0) for Light Vehicle (mcClass: 1) and Heavy Vehicle (mcClass: 2)
			laneApprMovs->get_Item(0l, &laneApprMov); // to South
			laneApprMov->get_LaneApproachMovementMCs(&laneApprMovMCs);
			laneApprMovMCs->get_Item(1l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);
			laneApprMovMCs->get_Item(2l, &laneApprMovMC);
			laneApprMovMC->put_Exists(true);

			wprintf(L"Updating Geometry Data...\n");

			// Call the method UpdateGeometryData() after Leg(s) and Lane(s) are added or removed.
			site->UpdateGeometryData(&boolResult);

			wprintf(L"Updated Geometry Data.\n");

			// Set Vehicle Origin-Destination (OD) Movements data, e.g. Volumes
			site->get_MovementVehicleODs(&movVehODs);
			movVehODs->get_Item(0l, 6l, &movVehOD);   // Vehicle OD Movement South to West

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(25.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(1.0f);

			movVehODs->get_Item(0l, 4l, &movVehOD);   // Vehicle OD Movement South to North

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(150.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(3.0f);

			movVehODs->get_Item(0l, 2l, &movVehOD);   // Vehicle OD Movement South to East

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(100.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(2.0f);

			movVehODs->get_Item(2l, 0l, &movVehOD);   // Vehicle OD Movement East to South

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(25.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(1.0f);

			movVehODs->get_Item(2l, 6l, &movVehOD);   // Vehicle OD Movement East to West

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(150.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(3.0f);

			movVehODs->get_Item(2l, 4l, &movVehOD);   // Vehicle OD Movement East to North

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(100.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(2.0f);

			movVehODs->get_Item(4l, 2l, &movVehOD);   // Vehicle OD Movement North to East

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(25.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(1.0f);

			movVehODs->get_Item(4l, 0l, &movVehOD);   // Vehicle OD Movement North to South

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(150.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(3.0f);

			movVehODs->get_Item(4l, 6l, &movVehOD);   // Vehicle OD Movement North to West

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(100.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(2.0f);

			movVehODs->get_Item(6l, 4l, &movVehOD);   // Vehicle OD Movement West to North

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(25.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(1.0f);

			movVehODs->get_Item(6l, 2l, &movVehOD);   // Vehicle OD Movement West to East

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(150.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(3.0f);

			movVehODs->get_Item(6l, 0l, &movVehOD);   // Vehicle OD Movement West to South

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(100.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(2.0f);

			// Set Pedestrian Movement Data
			site->get_MovementPeds(&movPeds);
			movPeds->get_Item(3l, 0l, 1l, &movPed); // Staged Crossing (Type: 3), South Leg (Origin: 0), Stage 1 (Approach Side)
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);
			movPeds->get_Item(3l, 0l, 2l, &movPed); // Staged Crossing (Type: 3), South Leg (Origin: 0), Stage 2 (Exit Side)
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);

			movPeds->get_Item(3l, 2l, 1l, &movPed); // Staged Crossing (Type: 3), East Leg (Origin: 2), Stage 1 (Approach Side)
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);
			movPeds->get_Item(3l, 2l, 2l, &movPed); // Staged Crossing (Type: 3), East Leg (Origin: 2), Stage 2 (Exit Side)
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);

			movPeds->get_Item(3l, 4l, 1l, &movPed);
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);
			movPeds->get_Item(3l, 4l, 2l, &movPed);
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);

			movPeds->get_Item(3l, 6l, 1l, &movPed);
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);
			movPeds->get_Item(3l, 6l, 2l, &movPed);
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);

			VARIANT_BOOL boolResult = false;

			wprintf(L"\nProcessing Site...\n");

			site->Process(&boolResult);
			if (boolResult)
			{
				CComPtr<ISIAPIOutputset> outputset = NULL;
				site->get_Outputset(&outputset);

				if (outputset == NULL)
					throw L"Error - No Site Output Data!";

				DATE generateDate = 0.0;
				outputset->get_Generated(&generateDate);

				COleDateTime oleDate = generateDate;

				wprintf(L"Processed Site at %02i/%02i/%i %02i:%02i:%02i.",
					oleDate.GetDay(), oleDate.GetMonth(), oleDate.GetYear(),
					oleDate.GetHour(), oleDate.GetMinute(), oleDate.GetSecond());

			}
			else
			{
				CComBSTR errStr = NULL;
				site->get_ProcessingError(&errStr);
				wprintf(L"%s\n", errStr);
			}

			wprintf(L"\nFinish successfully.\n");

		}
		catch (wchar_t* param)
		{
			wprintf(param);
		}
		catch (...)
		{
			wprintf(L"Error!");
		}

		pISIAPI->CloseProject(&boolResult);

		return 0;
	}

	// A sample to create a Roundabout Site, call the method SetDefaultLaneDisciplines() to automatically set the default Lane Disciplines on all Approach Lanes
	int CppSIAPITest::CreateRoundaboutSite2()
	{
		VARIANT_BOOL boolResult = false;
		
		try
		{
			pISIAPI->CreateAndOpenProject(CComBSTR("C:\\Temp\\Roundabout2.sip8"), CComBSTR("Roundabout2"), &boolResult);

			CComPtr<ISIAPIProject> project;
			pISIAPI->get_Project(&project);

			CComBSTR name;
			project->get_Name(&name);
			wprintf(L"\nProject '%s' is created.\n", name);

			CComPtr<ISIAPISite> site;
			project->AddSite(1l, 4l, &site);

			CComBSTR siteName("Roundabout2");
			site->put_Name(siteName);

			wprintf(L"Created Roundabout Site '%s'.\n", siteName);

			CComPtr<ISIAPILegs> legs;
			CComPtr<ISIAPILeg> leg;
			CComPtr<ISIAPILeg_roundabout> legRou;
			//CComPtr<ISIAPILaneApproachs> laneApprs;
			CComPtr<ISIAPILaneApproach> laneAppr;
			CComPtr<ISIAPILaneApproachMovements> laneApprMovs;
			CComPtr<ISIAPILaneApproachMovement> laneApprMov;
			CComPtr<ISIAPILaneApproachMovementMCs> laneApprMovMCs;
			CComPtr<ISIAPILaneApproachMovementMC> laneApprMovMC;
			CComPtr<ISIAPIIsland> island;
			CComPtr<ISIAPILaneExit> laneExit;
			CComPtr<ISIAPILaneSegment> laneSeg;
			CComPtr<ISIAPILaneMovements> laneMovs;
			CComPtr<ISIAPILaneMovement> laneMov;
			CComPtr<ISIAPILaneMovementMCs> laneMovMCs;
			CComPtr<ISIAPILaneMovementMC> laneMovMC;
			CComPtr<ISIAPIMovement_vehicle_ods> movVehODs;
			CComPtr<ISIAPIMovement_vehicle_od> movVehOD;
			CComPtr<ISIAPIMovement_vehicle_od_mcs> movVehODMCs;
			CComPtr<ISIAPIMovement_vehicle_od_mc> movVehODMC;
			CComPtr<ISIAPIMovement_peds> movPeds;
			CComPtr<ISIAPIMovement_ped> movPed;
			long legItemPosition = 1; // for Leg_item Position

			// Add a Leg on South
			site->AddLeg(0l, &leg);
			leg->put_Name(CComBSTR("Eley Road"));

			// Add three Exit Lanes
			leg->InsertExitLane(legItemPosition++, &laneExit);
			leg->InsertExitLane(legItemPosition++, &laneExit);
			leg->InsertExitLane(legItemPosition++, &laneExit);

			// A Roundabout Splitter Island must be added between Approach Lanes and Exit Lanes on each Two-way Leg on a Roundabout Site.
			// A normal Strip Island can be added (by calling the method leg.InsertIsland()) between two Approach Lanes or between two Exit Lanes if required for the geometry layout.
			leg->InsertIsland_RoundaboutSplitter(legItemPosition++, &island);

			// Add three Approach Lanes
			leg->InsertApproachLane(legItemPosition++, &laneAppr);
			leg->InsertApproachLane(legItemPosition++, &laneAppr);
			leg->InsertApproachLane(legItemPosition++, &laneAppr);

			// Change the Roundabout data on the Leg
			leg->get_Leg_roundabout(&legRou);
			legRou->put_Num_circulating_lanes(2l);
			legRou->put_Circulating_width(10.0f);
			legRou->put_Island_diameter(36.0f);

			// Add a Leg on East
			site->AddLeg(2l, &leg);
			leg->put_Name(CComBSTR("Park Road"));

			legItemPosition = 1; // reset the Leg_item position to 1

			// Add two Exit Lanes
			leg->InsertExitLane(legItemPosition++, &laneExit);
			leg->InsertExitLane(legItemPosition++, &laneExit);

			// Add a Roundabout Splitter Island
			leg->InsertIsland_RoundaboutSplitter(legItemPosition++, &island);

			// Add two Approach Lanes
			leg->InsertApproachLane(legItemPosition++, &laneAppr);
			leg->InsertApproachLane(legItemPosition++, &laneAppr);

			// Change the Roundabout data on the Leg
			leg->get_Leg_roundabout(&legRou);
			legRou->put_Num_circulating_lanes(3l);
			legRou->put_Circulating_width(12.0f);
			legRou->put_Island_diameter(36.0f);

			// Add a Leg on North
			site->AddLeg(4l, &leg);
			leg->put_Name(CComBSTR("Eley Road"));

			legItemPosition = 1; // reset the Leg_item position to 1

			// Add three Exit Lanes
			leg->InsertExitLane(legItemPosition++, &laneExit);
			leg->InsertExitLane(legItemPosition++, &laneExit);
			leg->InsertExitLane(legItemPosition++, &laneExit);

			// Add a Roundabout Splitter Island
			leg->InsertIsland_RoundaboutSplitter(legItemPosition++, &island);

			// Add three Approach Lanes
			leg->InsertApproachLane(legItemPosition++, &laneAppr);
			leg->InsertApproachLane(legItemPosition++, &laneAppr);
			leg->InsertApproachLane(legItemPosition++, &laneAppr);

			// Change the Roundabout data on the Leg
			leg->get_Leg_roundabout(&legRou);
			legRou->put_Num_circulating_lanes(2l);
			legRou->put_Circulating_width(10.0f);
			legRou->put_Island_diameter(36.0f);

			// Add a Leg on West
			site->AddLeg(6l, &leg);
			leg->put_Name(CComBSTR("Park Road"));

			legItemPosition = 1; // reset the Leg_item position to 1

			// Add two Exit Lanes
			leg->InsertExitLane(legItemPosition++, &laneExit);
			leg->InsertExitLane(legItemPosition++, &laneExit);

			// Add a Roundabout Splitter Island
			leg->InsertIsland_RoundaboutSplitter(legItemPosition++, &island);

			// Add two Approach Lanes
			leg->InsertApproachLane(legItemPosition++, &laneAppr);
			leg->InsertApproachLane(legItemPosition++, &laneAppr);

			// Change the Roundabout data on the Leg
			leg->get_Leg_roundabout(&legRou);
			legRou->put_Num_circulating_lanes(3l);
			legRou->put_Circulating_width(12.0f);
			legRou->put_Island_diameter(36.0f);

			// Set the default Lane Disciplines on all Approach Lanes on this Site
			pCppSIAPI->SetDefaultLaneDisciplines(site);

			// Customise the Lane Disciplines here if required...
			// after that, call site->UpdateGeometryData()

			wprintf(L"Updating Geometry Data...\n");

			// Call the method UpdateGeometryData() after Leg(s) and Lane(s) are added or removed.
			site->UpdateGeometryData(&boolResult);

			wprintf(L"Updated Geometry Data.\n");

			// Set Vehicle Origin-Destination (OD) Movements data, e.g. Volumes
			site->get_MovementVehicleODs(&movVehODs);
			movVehODs->get_Item(0l, 6l, &movVehOD);   // Vehicle OD Movement South to West

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(25.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(1.0f);

			movVehODs->get_Item(0l, 4l, &movVehOD);   // Vehicle OD Movement South to North

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(150.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(3.0f);

			movVehODs->get_Item(0l, 2l, &movVehOD);   // Vehicle OD Movement South to East

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(100.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(2.0f);

			movVehODs->get_Item(2l, 0l, &movVehOD);   // Vehicle OD Movement East to South

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(25.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(1.0f);

			movVehODs->get_Item(2l, 6l, &movVehOD);   // Vehicle OD Movement East to West

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(150.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(3.0f);

			movVehODs->get_Item(2l, 4l, &movVehOD);   // Vehicle OD Movement East to North

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(100.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(2.0f);

			movVehODs->get_Item(4l, 2l, &movVehOD);   // Vehicle OD Movement North to East

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(25.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(1.0f);

			movVehODs->get_Item(4l, 0l, &movVehOD);   // Vehicle OD Movement North to South

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(150.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(3.0f);

			movVehODs->get_Item(4l, 6l, &movVehOD);   // Vehicle OD Movement North to West

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(100.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(2.0f);

			movVehODs->get_Item(6l, 4l, &movVehOD);   // Vehicle OD Movement West to North

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(25.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(1.0f);

			movVehODs->get_Item(6l, 2l, &movVehOD);   // Vehicle OD Movement West to East

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(150.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(3.0f);

			movVehODs->get_Item(6l, 0l, &movVehOD);   // Vehicle OD Movement West to South

			movVehOD->get_MovementVehicleODMCs(&movVehODMCs);
			movVehODMCs->get_Item(1l, &movVehODMC);  // Light Vehicle Movement
			movVehODMC->put_Volume(100.0f);

			movVehODMCs->get_Item(2l, &movVehODMC);  // Heavy Vehicle Movement
			movVehODMC->put_Volume(2.0f);

			// Set Pedestrian Movement Data
			site->get_MovementPeds(&movPeds);
			movPeds->get_Item(3l, 0l, 1l, &movPed); // Staged Crossing (Type: 3), South Leg (Origin: 0), Stage 1 (Approach Side)
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);
			movPeds->get_Item(3l, 0l, 2l, &movPed); // Staged Crossing (Type: 3), South Leg (Origin: 0), Stage 2 (Exit Side)
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);

			movPeds->get_Item(3l, 2l, 1l, &movPed); // Staged Crossing (Type: 3), East Leg (Origin: 2), Stage 1 (Approach Side)
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);
			movPeds->get_Item(3l, 2l, 2l, &movPed); // Staged Crossing (Type: 3), East Leg (Origin: 2), Stage 2 (Exit Side)
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);

			movPeds->get_Item(3l, 4l, 1l, &movPed);
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);
			movPeds->get_Item(3l, 4l, 2l, &movPed);
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);

			movPeds->get_Item(3l, 6l, 1l, &movPed);
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);
			movPeds->get_Item(3l, 6l, 2l, &movPed);
			movPed->put_Exists(true);
			movPed->put_Volume(25.0f);

			VARIANT_BOOL boolResult = false;

			wprintf(L"\nProcessing Site...\n");

			site->Process(&boolResult);
			if (boolResult)
			{
				CComPtr<ISIAPIOutputset> outputset = NULL;
				site->get_Outputset(&outputset);

				if (outputset == NULL)
					throw L"Error - No Site Output Data!";

				DATE generateDate = 0.0;
				outputset->get_Generated(&generateDate);

				COleDateTime oleDate = generateDate;

				wprintf(L"Processed Site at %02i/%02i/%i %02i:%02i:%02i.",
					oleDate.GetDay(), oleDate.GetMonth(), oleDate.GetYear(),
					oleDate.GetHour(), oleDate.GetMinute(), oleDate.GetSecond());

			}
			else
			{
				CComBSTR errStr = NULL;
				site->get_ProcessingError(&errStr);
				wprintf(L"%s\n", errStr);
			}

			wprintf(L"\nFinish successfully.\n");

		}
		catch (wchar_t* param)
		{
			wprintf(param);
		}
		catch (...)
		{
			wprintf(L"Error!");
		}

		pISIAPI->CloseProject(&boolResult);

		return 0;
	}
}

